/*******************************************************************************
 * Copyright 2018 A-SIT Plus GmbH
 * AT-specific eIDAS Connector has been developed in a cooperation between EGIZ, 
 * A-SIT Plus GmbH, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "License");
 * You may not use this work except in compliance with the License.
 * You may obtain a copy of the License at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
package at.asitplus.eidas.specific.connector.config;

import java.util.List;

import org.opensaml.saml2.metadata.ContactPerson;
import org.opensaml.saml2.metadata.Organization;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import at.asitplus.eidas.specific.connector.MSeIDASNodeConstants;
import at.gv.egiz.eaaf.core.api.idp.IConfiguration;
import at.gv.egiz.eaaf.core.exceptions.EAAFException;
import at.gv.egiz.eaaf.modules.pvp2.api.IPVP2BasicConfiguration;

@Service("PVPEndPointConfiguration")
public class PVPEndPointConfiguration implements IPVP2BasicConfiguration {
	private static final Logger log = LoggerFactory.getLogger(PVPEndPointConfiguration.class);
	
	@Autowired(required=true) IConfiguration basicConfiguration;
	
	@Override
	public String getIDPEntityId(String authURL) throws EAAFException {
		return removePostFix(authURL) + MSeIDASNodeConstants.ENDPOINT_PVP_METADATA;
		
	} 

	@Override 
	public String getIDPSSOPostService(String authURL) throws EAAFException {
		return removePostFix(authURL) + MSeIDASNodeConstants.ENDPOINT_PVP_POST;
		
	}

	@Override
	public String getIDPSSORedirectService(String authURL) throws EAAFException {
		return removePostFix(authURL) + MSeIDASNodeConstants.ENDPOINT_PVP_REDIRECT;
		
	}

	@Override
	public Object getIDPSSOSOAPService(String extractAuthURLFromRequest) throws EAAFException {
		log.warn("PVP S-Profile End-Point does NOT support SOAP Binding");
		return null;
		
	}

	@Override
	public List<ContactPerson> getIDPContacts() throws EAAFException {
		// Return contact person, if it shall be included in metadata
		return null;
	}

	@Override
	public Organization getIDPOrganisation() throws EAAFException {
		// Return organization, if it shall be included in metadata
		return null;
	}

	private String removePostFix(String url) {
		if (url != null && url.endsWith("/"))
			return url.substring(0, url.length() - 1);
		else
			return url;
	}
}
