/*******************************************************************************
 * Copyright 2018 A-SIT Plus GmbH
 * AT-specific eIDAS Connector has been developed in a cooperation between EGIZ, 
 * A-SIT Plus GmbH, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "License");
 * You may not use this work except in compliance with the License.
 * You may obtain a copy of the License at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
package at.asitplus.eidas.specific.connector.logger;

import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import at.asitplus.eidas.specific.connector.MSeIDASNodeConstants;
import at.gv.egiz.eaaf.core.api.IRequest;
import at.gv.egiz.eaaf.core.api.idp.IAuthData;
import at.gv.egiz.eaaf.core.api.logging.IStatisticLogger;
import at.gv.egiz.eaaf.core.exceptions.EAAFException;

public class StatisticLogger implements IStatisticLogger {

	private static final Logger log = LoggerFactory.getLogger(StatisticLogger.class);
	
	private static final String DATEFORMATER = "yyyy.MM.dd-HH:mm:ss+z";
	private static final String STATUS_SUCCESS = "success";
	private static final String STATUS_ERROR = "error";
	
	@Override
	public void logSuccessOperation(IRequest protocolRequest, IAuthData authData, boolean isSSOSession) {		
		log.info(buildLogMessage(
				protocolRequest.getUniqueTransactionIdentifier(), 
				protocolRequest.getSPEntityId(), 
				protocolRequest.getRawData(MSeIDASNodeConstants.DATA_REQUESTERID), 
				protocolRequest.getServiceProviderConfiguration().getAreaSpecificTargetIdentifier(), 
				authData.getCiticenCountryCode(), 
				STATUS_SUCCESS , 
				StringUtils.EMPTY, 
				StringUtils.EMPTY));
		

	} 
 
	@Override
	public void logErrorOperation(Throwable throwable) {		
		String errorId = "TODO";
		if (throwable instanceof EAAFException)
			errorId = ((EAAFException) throwable).getErrorId();
						
		log.info(buildLogMessage(
				StringUtils.EMPTY, 
				StringUtils.EMPTY, 
				StringUtils.EMPTY, 
				StringUtils.EMPTY,
				StringUtils.EMPTY, 
				STATUS_ERROR, 
				errorId, 
				throwable.getMessage()));
		

	}

	@Override
	public void logErrorOperation(Throwable throwable, IRequest errorRequest) {
		String errorId = "TODO";
		if (throwable instanceof EAAFException)
			errorId = ((EAAFException) throwable).getErrorId();
		
		if (errorRequest != null)	
			log.info(buildLogMessage(
					errorRequest.getUniqueTransactionIdentifier(), 
					errorRequest.getSPEntityId(), 
					errorRequest.getRawData(MSeIDASNodeConstants.DATA_REQUESTERID), 
					errorRequest.getServiceProviderConfiguration().getAreaSpecificTargetIdentifier(), 
					StringUtils.EMPTY, 
					STATUS_ERROR, 
					errorId, 
					throwable.getMessage()));
		
		else
			log.info(buildLogMessage(
					StringUtils.EMPTY, 
					StringUtils.EMPTY, 
					StringUtils.EMPTY, 
					StringUtils.EMPTY,
					StringUtils.EMPTY, 
					STATUS_ERROR, 
					errorId, 
					throwable.getMessage()));

	}

	@Override
	public void internalTesting() throws Exception {
		log.trace("Not implemented for a File-based logger");

	}

	private String buildLogMessage(String tId, String moaIDEntityId, Object requesterId, String target, String cc, 
			String status, String errorCode, String errorMsg) {
		String logMsg = StringUtils.EMPTY;
		
		//data,tId,MOAID-Id,SP-Id,bPKTarget,CC,status,error-code,error-msg
		
		logMsg += DateTime.now().toString(DATEFORMATER ) + ",";
		logMsg += tId + ",";
		logMsg += moaIDEntityId  + ",";
		
		if (requesterId instanceof String && StringUtils.isNotEmpty((String)requesterId))
			logMsg += (String)requesterId + ",";
		else
			logMsg += StringUtils.EMPTY + ",";
				
		logMsg += target  + ",";		
		logMsg += cc + ",";
				
		logMsg += status + ",";
		logMsg += errorCode + ",";
		logMsg += errorMsg;
		
		return logMsg;
	}
}
