/*******************************************************************************
 * Copyright 2018 A-SIT Plus GmbH
 * AT-specific eIDAS Connector has been developed in a cooperation between EGIZ, 
 * A-SIT Plus GmbH, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "License");
 * You may not use this work except in compliance with the License.
 * You may obtain a copy of the License at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
package at.asitplus.eidas.specific.connector.processes.tasks;

import java.util.Enumeration;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringEscapeUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import at.asitplus.eidas.specific.connector.MSConnectorEventCodes;
import at.asitplus.eidas.specific.connector.MSeIDASNodeConstants;
import at.gv.egiz.eaaf.core.api.data.EAAFConstants;
import at.gv.egiz.eaaf.core.api.idp.process.ExecutionContext;
import at.gv.egiz.eaaf.core.exceptions.TaskExecutionException;
import at.gv.egiz.eaaf.core.impl.idp.auth.modules.AbstractAuthServletTask;

/**
 * @author tlenz
 *
 */
@Component("EvaluateCountrySelectionTask")
public class EvaluateCountrySelectionTask extends AbstractAuthServletTask {
	private static final Logger log = LoggerFactory.getLogger(EvaluateCountrySelectionTask.class);
	
	
	@Override
	public void execute(ExecutionContext executionContext, HttpServletRequest request, HttpServletResponse response)
			throws TaskExecutionException {		
		try {
			
			String stopAuthFlag = request.getParameter(MSeIDASNodeConstants.REQ_PARAM_STOP_PROCESS);
			if (StringUtils.isNotEmpty(stopAuthFlag) && Boolean.parseBoolean(stopAuthFlag)) {
				log.info("Authentication process WAS stopped by entity. Stopping auth. process ... ");
				revisionsLogger.logEvent(pendingReq, MSConnectorEventCodes.PROCESS_STOPPED_BY_USER);
				pendingReq.setAbortedByUser(true);
				pendingReq.setAuthenticated(false);
				
			} else {	 					 
				// set parameter execution context			
				Enumeration<String> reqParamNames = request.getParameterNames();
				while(reqParamNames.hasMoreElements()) {
					String paramName = reqParamNames.nextElement();
					if (StringUtils.isNotEmpty(paramName) && 
							!EAAFConstants.PROCESS_ENGINE_PENDINGREQUESTID.equalsIgnoreCase(paramName)) {
						for (String el : MSeIDASNodeConstants.COUNTRY_SELECTION_PARAM_WHITELIST) {
							if (el.equalsIgnoreCase(paramName))
								executionContext.put(paramName, 
										StringEscapeUtils.escapeHtml(request.getParameter(paramName)));
						}
					}
				}    	    			
				log.info("Country selection finished. Starting auth. process for country ... ");
				
			}
	    	    			
		} catch (Exception e) {
			log.warn("EvaluateBKUSelectionTask has an internal error", e);
			throw new TaskExecutionException(pendingReq, e.getMessage(), e);
			
		}
	}

}
