/*******************************************************************************
 * Copyright 2018 A-SIT Plus GmbH
 * AT-specific eIDAS Connector has been developed in a cooperation between EGIZ, 
 * A-SIT Plus GmbH, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "License");
 * You may not use this work except in compliance with the License.
 * You may obtain a copy of the License at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
package at.asitplus.eidas.specific.connector.processes.tasks;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import at.asitplus.eidas.specific.connector.MSConnectorEventCodes;
import at.asitplus.eidas.specific.connector.MSeIDASNodeConstants;
import at.asitplus.eidas.specific.connector.gui.StaticGuiBuilderConfiguration;
import at.gv.egiz.eaaf.core.api.gui.IGUIBuilderConfiguration;
import at.gv.egiz.eaaf.core.api.gui.IGUIFormBuilder;
import at.gv.egiz.eaaf.core.api.idp.IConfiguration;
import at.gv.egiz.eaaf.core.api.idp.process.ExecutionContext;
import at.gv.egiz.eaaf.core.exceptions.EAAFException;
import at.gv.egiz.eaaf.core.exceptions.GUIBuildException;
import at.gv.egiz.eaaf.core.exceptions.TaskExecutionException;
import at.gv.egiz.eaaf.core.impl.idp.auth.modules.AbstractAuthServletTask;

/**
 * @author tlenz
 *
 */
@Component("GenerateCountrySelectionFrameTask")
public class GenerateCountrySelectionFrameTask extends AbstractAuthServletTask {
	private static final Logger log = LoggerFactory.getLogger(GenerateCountrySelectionFrameTask.class);
	
	@Autowired IGUIFormBuilder guiBuilder;
	@Autowired IConfiguration basicConfig;
	
	@Override
	public void execute(ExecutionContext executionContext, HttpServletRequest request, HttpServletResponse response)
			throws TaskExecutionException {
		try {			
			revisionsLogger.logEvent(pendingReq, MSConnectorEventCodes.STARTING_COUNTRY_SELECTION);
																					
			IGUIBuilderConfiguration config = new StaticGuiBuilderConfiguration(
					basicConfig, 
					pendingReq, 
					MSeIDASNodeConstants.TEMPLATE_HTML_COUNTRYSELECTION, 
					MSeIDASNodeConstants.ENDPOINT_COUNTRYSELECTION); 
					 			 
			guiBuilder.build(response, config, "BKU-Selection form");
			
		} catch (GUIBuildException e) {	
			log.warn("Can not build GUI:'BKU-Selection'. Msg:" + e.getMessage());
			throw new TaskExecutionException(pendingReq, 
					"Can not build GUI. Msg:" + e.getMessage(), 
					new EAAFException("gui.00", new Object[]{e.getMessage()}, e));
			
		} catch (Exception e) {
			log.warn("FinalizeAuthenticationTask has an internal error", e);
			throw new TaskExecutionException(pendingReq, e.getMessage(), e);
			
		}
	
	}

}
