/*******************************************************************************
 * Copyright 2018 A-SIT Plus GmbH
 * AT-specific eIDAS Connector has been developed in a cooperation between EGIZ, 
 * A-SIT Plus GmbH, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "License");
 * You may not use this work except in compliance with the License.
 * You may obtain a copy of the License at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
/*******************************************************************************
 *******************************************************************************/
package at.asitplus.eidas.specific.connector.provider;

import java.net.MalformedURLException;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import at.asitplus.eidas.specific.connector.MSeIDASNodeConstants;
import at.gv.egiz.eaaf.core.api.idp.IConfiguration;
import at.gv.egiz.eaaf.core.exceptions.EAAFConfigurationException;
import at.gv.egiz.eaaf.core.exceptions.EAAFException;
import at.gv.egiz.eaaf.core.impl.utils.FileUtils;
import at.gv.egiz.eaaf.modules.pvp2.impl.utils.AbstractCredentialProvider;

public class PVPEndPointCredentialProvider extends AbstractCredentialProvider {
	private static final Logger log = LoggerFactory.getLogger(PVPEndPointCredentialProvider.class);
	
	@Autowired(required=true) IConfiguration basicConfiguration;
	
	@Override
	public String getFriendlyName() {
		return "PVP2 S-Profile EndPoint";
	}

	@Override
	public String getKeyStoreFilePath() throws EAAFException {
		try {
			String path = basicConfiguration.getBasicConfiguration(MSeIDASNodeConstants.PROP_CONFIG_PVP2_KEYSTORE_PATH);
			if (StringUtils.isEmpty(path)) {
				log.error(getFriendlyName() + " | Path to keyStore is NULL or EMPTY");
				throw new EAAFConfigurationException("config.27", 
						new Object[] {getFriendlyName() + " | Path to keyStore is NULL or EMPTY"});
				
			}
			 
			return FileUtils.makeAbsoluteURL( 
					path, 
					basicConfiguration.getConfigurationRootDirectory());
			
		} catch (MalformedURLException e) {
			log.error(getFriendlyName() + " | Path to keyStore NOT valid.", e);
			throw new EAAFConfigurationException("config.27", 
					new Object[] {getFriendlyName() + " | Path to keyStore NOT valid."}, e);
			
		}
		
	}

	@Override
	public String getKeyStorePassword() {
		return basicConfiguration.getBasicConfiguration(MSeIDASNodeConstants.PROP_CONFIG_PVP2_KEYSTORE_PASSWORD);
		
	}

	@Override
	public String getMetadataKeyAlias() {
		return basicConfiguration.getBasicConfiguration(MSeIDASNodeConstants.PROP_CONFIG_PVP2_KEY_METADATA_ALIAS);
	}

	@Override
	public String getMetadataKeyPassword() {
		return basicConfiguration.getBasicConfiguration(MSeIDASNodeConstants.PROP_CONFIG_PVP2_KEY_METADATA_PASSWORD);
		
	}

	@Override
	public String getSignatureKeyAlias() {
		return basicConfiguration.getBasicConfiguration(MSeIDASNodeConstants.PROP_CONFIG_PVP2_KEY_SIGNING_ALIAS);
		
	}

	@Override
	public String getSignatureKeyPassword() {
		return basicConfiguration.getBasicConfiguration(MSeIDASNodeConstants.PROP_CONFIG_PVP2_KEY_SIGNING_PASSWORD);
		
	}

	@Override
	public String getEncryptionKeyAlias() {
		return null;
		
	}

	@Override
	public String getEncryptionKeyPassword() {
		return null;
		
	}

}
