/*******************************************************************************
 * Copyright 2018 A-SIT Plus GmbH
 * AT-specific eIDAS Connector has been developed in a cooperation between EGIZ, 
 * A-SIT Plus GmbH, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "License");
 * You may not use this work except in compliance with the License.
 * You may obtain a copy of the License at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
package at.asitplus.eidas.specific.modules.authmodule_eIDASv2.handler;

import java.util.Base64;

import org.apache.commons.codec.DecoderException;
import org.apache.commons.codec.binary.Hex;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import at.asitplus.eidas.specific.modules.authmodule_eIDASv2.Constants;
import at.asitplus.eidas.specific.modules.authmodule_eIDASv2.exception.eIDASAttributeException;
import at.asitplus.eidas.specific.modules.authmodule_eIDASv2.exception.eIDPostProcessingException;
import at.asitplus.eidas.specific.modules.authmodule_eIDASv2.utils.eIDASResponseUtils;
import at.gv.egiz.eaaf.core.impl.data.Trible;

public class DEeIDPostProcessor extends AbstracteIDPostProcessor {
	private static final Logger log = LoggerFactory.getLogger(DEeIDPostProcessor.class);	
	private static final String canHandleCC = "DE";
	
	private int priority = 1;
	
	@Override
	public int getPriority() {
		return priority;
	}

	@Override
	public boolean canHandle(String countryCode) {		
		return countryCode != null && countryCode.equalsIgnoreCase(canHandleCC);
		
	}

	
	public void setPriority(int priority) {
		this.priority = priority;
	}

	@Override
	public String getName() {
		return "DE-PostProcessor";
		
	}
	
	@Override
	protected String processPseudonym(Object eIdentifierObj) throws eIDPostProcessingException, eIDASAttributeException {
		if (eIdentifierObj == null || !(eIdentifierObj instanceof String))
	        	throw new eIDASAttributeException(Constants.eIDAS_ATTR_PERSONALIDENTIFIER);
		 
		Trible<String, String, String> eIdentifier = 
					eIDASResponseUtils.parseEidasPersonalIdentifier((String)eIdentifierObj);
				
		log.trace(getName() + " starts processing of attribute: " + Constants.eIDAS_ATTR_PERSONALIDENTIFIER);
		String result = convertDEIdentifier(eIdentifier.getThird());
		log.debug(getName() + " finished processing of attribute: " + Constants.eIDAS_ATTR_PERSONALIDENTIFIER);
		
		return result; 
		
		
	}
	
	private String convertDEIdentifier(String hexEncodedDEIdentifier) throws eIDPostProcessingException
	{
		if(hexEncodedDEIdentifier.length() != 64)
			throw new eIDPostProcessingException("ernb.03", new Object[] {"Input has wrong length, expected 64 chars"});

		byte[] data;
		try {			
			data = Hex.decodeHex(hexEncodedDEIdentifier);
			
		} catch (DecoderException e) {
			throw new eIDPostProcessingException("ernb.03", null, e);
			
		}
		
		byte[] encoded = Base64.getEncoder().encode(data);
		return new String(encoded);
	}

}
