/*******************************************************************************
 * Copyright 2018 A-SIT Plus GmbH
 * AT-specific eIDAS Connector has been developed in a cooperation between EGIZ, 
 * A-SIT Plus GmbH, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "License");
 * You may not use this work except in compliance with the License.
 * You may obtain a copy of the License at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
package at.asitplus.eidas.specific.modules.authmodule_eIDASv2.handler;

import java.util.Map;

import at.asitplus.eidas.specific.modules.authmodule_eIDASv2.DAO.ERnBeIDData;
import at.asitplus.eidas.specific.modules.authmodule_eIDASv2.exception.eIDASAttributeException;
import at.asitplus.eidas.specific.modules.authmodule_eIDASv2.exception.eIDPostProcessingException;

public interface INationaleIDPostProcessor {

	/**
	 * Get a friendlyName of this post-processor implementation
	 * 
	 * @return
	 */
	public String getName();
	
	/**
	 * Get the priority of this eID Post-Processor
	 * <br>
	 * If more than one Post-Processor implementations can handle the eID data, the post-processor with the highest priority are selected.
	 * The Default-Processor has priority '0'
	 * 
	 * @return Priority of this handler
	 */
	public int getPriority();

	/**
	 * Check if this postProcessor is sensitive for a specific country 
	 * 
	 * @param countryCode of the eID data that should be processed 
	 * @return true if this implementation can handle the country, otherwise false
	 * 
	 */
	public boolean canHandle(String countryCode);
	
	
	/**
	 * Post-Process eIDAS eID data into national format 
	 * @param eIDASAttrMap Map of eIDAS attributes in format friendlyName and attribute
	 * @throws eIDPostProcessingException 
	 * @throws eIDASAttributeException 
	 * 
	 */
	public ERnBeIDData postProcess(Map<String, Object> eIDASAttrMap) throws eIDPostProcessingException, eIDASAttributeException; 
}
